require 'helper'

describe Responding do
  describe 'init' do
    it 'fails w/o attributes' do
      error = assert_raises ArgumentError do
        Responding.new
      end

      assert_match(/attribute/, error.message)
    end

    it 'returns a class object when attributes given' do
      assert_kind_of Class, Responding.new(:attr1)
    end
  end

  describe 'with plain attributes' do
    it 'simply adds accessors to the class' do
      cls = Responding.new(:name, :age)

      obj = cls.new

      refute obj.age
      obj.age = 22
      assert_equal 22, obj.age

      refute obj.name
      obj.name = "Steve"
      assert_equal "Steve", obj.name
    end
  end

  describe 'with hash attributes' do
    it 'nests the accessors into separate classes' do
      cls = Responding.new(:age, :name => [:first, :last])

      obj = cls.new

      refute obj.age
      obj.age = 22
      assert_equal 22, obj.age

      assert obj.name, obj.inspect
      refute obj.name.first, obj.name.inspect
      refute obj.name.last, obj.name.inspect
      obj.name.first = 'Steve'
      assert_equal 'Steve', obj.name.first
    end
  end

  describe 'constructor' do
    it 'accepts a hash as argument' do
      cls = Responding.new(:name, :age)

      obj = cls.new(:name => 'Bob', :age => 33)

      assert_equal 'Bob', obj.name
      assert_equal 33, obj.age
    end

    it 'accepts also a nested hash as argument' do
      cls = Responding.new(:age, :name => [:first, :last])

      obj = cls.new(:name => {:first => 'Bob', :last => 'Smith'}, :age => 44)

      assert_equal 'Bob', obj.name.first
      assert_equal 'Smith', obj.name.last
      assert_equal 44, obj.age
    end
  end

  describe 'nested accessor' do
    it 'delegates to the class constructor' do
      cls = Responding.new(:age, :name => [:first])

      obj = cls.new(:name => {:first => 'Bob'}, :age => 44)

      obj.name = {:first => 'Steve'}
      assert_equal 'Steve', obj.name.first
    end
  end
end
